/* pp_arm.h */

#include "gpio.h"

/* RX interface definitions */

#define HW_RX_OUT		(1 << 0)
#define HW_RX_SHIFT		(1 << 1)
#define HW_RX_DATA_O	(1 << 2)
#define HW_RX_DONE		(1 << 3)
#define HW_RX_TR		(1 << 4)
#define HW_RX_ERROR		(1 << 5)
#define HW_RX_AC_LO		(1 << 6)
#define HW_RX_DMA		(1 << 7)
#define HW_RX_12BIT		(1 << 8)
#define HW_RX_DATA_I	(1 << 9)
#define HW_RX_INIT		(1 << 10)
#define HW_RX_RUN		(1 << 11)

#define HW_RX_INITFF    (1 << 12)	// no hardware
#define HW_RX_LED1  	(1 << 13)	// no hardware
#define HW_RX_LED2  	(1 << 14)	// no hardware


#define RX_ERROR      (HW_RX_ERROR)
#define RX_TR         (HW_RX_TR)
#define RX_DONE       (HW_RX_DONE)
#define RX_SHIFT      (HW_RX_SHIFT)
#define RX_OUT        (HW_RX_OUT)
#define RX_AC_LO      (HW_RX_AC_LO)
//#define RX_INITFF     (HW_RX_INITFF)

#define RX_LED1       (HW_RX_LED1)
#define RX_LED2       (HW_RX_LED2)

#define RX_DATA_O     (HW_RX_DATA_O)

#define RX_DATA_I     (HW_RX_DATA_I)
#define RX_RUN        (HW_RX_RUN)
#define RX_INIT       (HW_RX_INIT)
#define RX_12BIT      (HW_RX_12BIT)
#define RX_DMA        (HW_RX_DMA)

inline static int __attribute__((always_inline)) rx_if_get_z(int mask, volatile int *gpio) {
    int i;
    int result = 0;

    for (i = 0; i < 12; i++) {
        if (io[i].sel & mask) {
            if (gpio[GPIO_DIN(io[i].bank, io[i].bit)] & GPIO_DIN_M(io[i].bank, io[i].bit))
                result |= io[i].sel;
            if (io[i].inv)
				result ^= io[i].sel;
		}
    }
    return result;
}

inline static int __attribute__((always_inline)) rx_if_get(int mask) {
	return rx_if_get_z(mask, gpio);
}

inline static void __attribute__((always_inline)) rx_if_put_z(int mask, int val, volatile int *gpio) {
    int i;
    int state;

    for (i = 0; i < 12; i++) {
        if (io[i].sel & mask) {
        	state = io[i].sel & val;
        	state = (io[i].inv) ? !state : state;
            if (state)
                gpio[GPIO_SET + GPIO_DOUT(io[i].bank, io[i].bit)] = GPIO_DOUT_M(io[i].bank, io[i].bit);
            else
                gpio[GPIO_CLR + GPIO_DOUT(io[i].bank, io[i].bit)] = GPIO_DOUT_M(io[i].bank, io[i].bit);
        }
    }        
}

inline static void __attribute__((always_inline)) rx_if_put(int mask, int val) {
	rx_if_put_z(mask, val, gpio);
}

inline static int __attribute__((always_inline)) rx_if_getff_z(int mask, volatile int *gpio) {
    int i;
    int result = 0;

    for (i = 0; i < 12; i++) {
        if (io[i].sel & mask)
            if (gpio[GPIO_INTSTAT(io[i].bank, io[i].bit)] & GPIO_INTSTAT_M(io[i].bank, io[i].bit))
                result |= io[i].sel;
    }
    return result;
}

inline static int __attribute__((always_inline)) rx_if_getff(int mask) {
	return rx_if_getff_z(mask, gpio);
}

inline static void __attribute__((always_inline)) rx_if_resetff_z(int mask, volatile int *gpio) {
    int i;

    for (i = 0; i < 12; i++) {
        if (io[i].sel & mask) {
	        gpio[GPIO_CLR + GPIO_INTSTAT(io[i].bank, io[i].bit)] = GPIO_INTSTAT_M(io[i].bank, io[i].bit);
        }
    }        
}

inline static void __attribute__((always_inline)) rx_if_resetff(int mask) {
	return rx_if_resetff_z(mask, gpio);
}

inline static void __attribute__((always_inline)) rx_if_set(int mask) {
	rx_if_put(mask, mask);
}

inline static void __attribute__((always_inline)) rx_if_clr(int mask) {
	rx_if_put(mask, 0);
};


extern int io_base;
extern int rx_if_image;
void log_ioput(int val, int mask);
void log_ioget(int val, int mask);
//int rx_if_get(int mask);
//void rx_if_put(int mask, int val);
//void rx_if_set(int mask);
//void rx_if_clr(int mask);
void rx_if_clrff(int mask);
int init_hwio(void);
