/* pp_netbsd.c */

#include "pp_netbsd.h"

u_long iomap[32];

#ifdef __NetBSD__

void outb(unsigned int port, u_int8_t val) {
   asm volatile( "out %0, %1"
                 : : "a"(val), "Nd"((u_int16_t) port) );
}

u_int8_t inb(unsigned int port) {
   u_int8_t ret;
   asm volatile( "in %1, %0"
                 : "=a"(ret) : "Nd"((u_int16_t) port) );
   return ret;
}

#endif


// RX interface bit I/O

// return the bits from the RX interface bit vector
// only those bits in mask are accurate, the rest are zeroed

int rx_if_getx(int mask)
{
    int val = 0;
    static int prev_val;
    static int prev_mask;
#ifdef FAST_IO
    int port_val;
    
    port_val = inl(IO_DATA) ^ (DATA_INV | (STATUS_INV << 8) | (CNTRL_INV << 16));
    val = (port_val & 0x0000ff) | ((port_val & 0x00ff00) << 8) | ((port_val & 0xff0000) >> 8);  
#elif READ_ALL
    val = ((inb(IO_STATUS) ^ STATUS_INV) << 16) | ((inb(IO_CNTRL) ^ CNTRL_INV) << 8) | (inb(IO_DATA) ^ DATA_INV);
#else
    if (mask & 0x0000ff) {
        val |= inb(IO_DATA) ^ DATA_INV;
    }
    if (mask & 0x00ff00) {
        val |= (inb(IO_CNTRL) ^ CNTRL_INV) << 8;
    }
    if (mask & 0xff0000) {
        val |= (inb(IO_STATUS) ^ STATUS_INV) << 16;
    }
#endif 
    if (val != prev_val || mask != prev_mask)
        log_ioget(val, mask);
    prev_val = val;
    prev_mask = mask;
    return val & mask;
}

int rx_if_get(int mask)
{
#if 1
    return rx_if_getx(mask);
#else
    int val1, val2, loops;

    loops = 0;
    do {
        val1 = rx_if_getx(mask);
        val2 = rx_if_getx(mask);
        loops += 1;
        if ((loops % 10) == 9)
            printf("rx_if_get: took 9 tries\n");
    }
    while (val1 != val2);
    if (loops > 2)
        printf("rx_if_get: took %d tries\n", loops);   
    return val1;
#endif
}

// change the state of mask bits to val

void rx_if_put(int mask, int val)
{
    static int dat;
#if defined __linux__
#ifdef FAST_IO
    int port_dat;

    dat &= ~mask;
    dat |= (mask & val);
    port_dat = (dat & 0x0000ff) | ((dat & 0x00ff00) << 8);
    port_dat ^= DATA_INV | (CNTRL_INV << 16);
    outl(port_dat, IO_DATA);
#else
    dat &= ~mask;
    dat |= (mask & val);
    if (mask & 0x000000ff)
        outb(dat ^ DATA_INV, IO_DATA);
    if (mask & 0x0000ff00)
        outb((dat >> 8) ^ CNTRL_INV, IO_CNTRL);
#endif
#elif defined __NetBSD__
#ifdef FAST_IO
    int port_dat;

    dat &= ~mask;
    dat |= (mask & val);
    port_dat = (dat & 0x0000ff) | ((dat & 0x00ff00) << 8);
    port_dat ^= DATA_INV | (CNTRL_INV << 16);
    outl(IO_DATA, port_dat);
#else
    dat &= ~mask;
    dat |= (mask & val);
    if (mask & 0x000000ff)
        outb(IO_DATA, dat ^ DATA_INV);
    if (mask & 0x0000ff00)
        outb(IO_CNTRL, (dat >> 8) ^ CNTRL_INV);
#endif
#endif
    rx_if_image = dat;
    log_ioput(rx_if_image, mask);
}

void rx_if_set(int mask)
{
     rx_if_put(mask, 0x00FFFFFF);
}

void rx_if_clr(int mask)
{
     rx_if_put(mask, 0x00000000); 
}

int init_hwio(void) {
	int retval;
	
    if (retval = i386_get_ioperm(iomap)) {
        perror("rxm: i386_get_ioperm");
        return retval;
    }
    iomap[IOMAP_OFFSET] &= ~IOMAP_BITMASK;
    if (retval = i386_set_ioperm(iomap)) {
        perror("rxm: i386_set_ioperm");
        return retval;
    }
    return 0;
}

