/* pp_netbsd.h */

#include <machine/pio.h>
#include <machine/sysarch.h>
int i386_get_ioperm(u_long *);
int i386_set_ioperm(u_long *);

/* fundamental definitions */

/* parallel port definitions */
/*   this is a retrofit of the address from being compile time to runtime */
#define IO_BASE	      (io_base)
#define IO_DATA        IO_BASE
#define IO_STATUS     (IO_BASE + 1)
#define IO_CNTRL      (IO_BASE + 2)
//#define FAST_IO                          /* use 32bit port access instructions instead of 8bit */

/* NetBSD port I/O permissions are handled by a bitmap */ 
#define IOMAP_OFFSET  (IO_BASE/32)
#ifndef FAST_IO
#define IOMAP_BITMASK ((1 << (IO_DATA%32)) | (1 << (IO_STATUS%32)) | (1 << (IO_CNTRL%32)))
#else
#define IOMAP_BITMASK (0x0F << (IO_DATA%32))
#endif
#define READ_ALL 0

#define BIT_D0 0001
#define BIT_D1 0002
#define BIT_D2 0004
#define BIT_D3 0010
#define BIT_D4 0020
#define BIT_D5 0040
#define BIT_D6 0100
#define BIT_D7 0200

#define BIT_C0 0001
#define BIT_C5 0040

#define BIT_S3 0010
#define BIT_S4 0020
#define BIT_S5 0040
#define BIT_S6 0100
#define BIT_S7 0200

/* RX hardware interface definitions */

#define HW_RX_REG_1      DATA      /* bits in register 1 */
#define HW_RX_ERROR      BIT_D0
#define HW_RX_TR         BIT_D1
#define HW_RX_DONE       BIT_D2
#define HW_RX_SHIFT      BIT_D3
#define HW_RX_OUT        BIT_D4
#define HW_RX_AC_LO      BIT_D5
#define HW_RX_LED1       BIT_D5
#define HW_RX_LED2       BIT_D6
#define HW_RX_INITFF     BIT_D7
#define HW_RX_REG_1_INV  (BIT_D7)
#define DATA_INV         HW_RX_REG_1_INV

#define HW_RX_REG_2      CNTRL     /* bits in register 2 */
#define HW_RX_DATA_O     BIT_C0
#define HW_RX_REG_2_INV  (BIT_C0)
#define CNTRL_INV        HW_RX_REG_2_INV

#define HW_RX_REG_3      STATUS    /* bits in register 3 */
#define HW_RX_RUN        BIT_S7  
#define HW_RX_DMA        BIT_S6
#define HW_RX_INIT       BIT_S5
#define HW_RX_DATA_I     BIT_S4
#define HW_RX_12BIT      BIT_S3 
#define HW_RX_REG_3_INV  (BIT_S3 | BIT_S4 | BIT_S6)
#define STATUS_INV       HW_RX_REG_3_INV

/* RX interface definitions */

#define RX_ERROR      (HW_RX_ERROR)
#define RX_TR         (HW_RX_TR)
#define RX_DONE       (HW_RX_DONE)
#define RX_SHIFT      (HW_RX_SHIFT)
#define RX_OUT        (HW_RX_OUT)
#define RX_AC_LO      (HW_RX_AC_LO)
#define RX_INITFF     (HW_RX_INITFF)

#define RX_LED1       (HW_RX_LED1)
#define RX_LED2       (HW_RX_LED2)

#define RX_DATA_O     (HW_RX_DATA_O << 8)

#define RX_DATA_I     (HW_RX_DATA_I << 16)
#define RX_RUN        (HW_RX_RUN << 16)
#define RX_INIT       (HW_RX_INIT << 16)
#define RX_12BIT      (HW_RX_12BIT << 16)
#define RX_DMA        (HW_RX_DMA << 16)

extern int io_base;
extern int rx_if_image;
void log_ioput(int val, int mask);
void log_ioget(int val, int mask);
int rx_if_get(int mask);
void rx_if_put(int mask, int val);
void rx_if_set(int mask);
void rx_if_clr(int mask);
int init_hwio(void);


